from pdb import set_trace

from django.contrib.syndication.feeds import FeedDoesNotExist, Feed
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
import settings

from models import GeneralPost, published
from utils import upcast
from views import get_tagged

def latest(cls, list_url_name):
    class PostsFeed(Feed):
        title = _('Latest posts')

        def get_object(self, bits):
            if len(bits) > 1:
                raise ObjectDoesNotExist
            if bits:
                return get_tagged(bits[0], published(cls.objects.all()))
            return None

        def categories(self, obj):
            if obj and len(obj) == 2:
                return obj[1]

        def link(self):
            return reverse(list_url_name)

        def items(self, obj):
            if obj:
                return obj[0][:20]
            return published(cls.objects.all())[:20]

        def item_pubdate(self, item):
            return item.publish_at

        def item_author_name(self, item):
            return item.author

        def item_categories(self, item):
            if settings.HAS_TAGGING:
                return item.get_tags()
    return PostsFeed

LatestPosts = latest(GeneralPost, 'dzenlog-post-list')



DEBUG = True
TEMPLATE_DEBUG = DEBUG

DATABASE_ENGINE = 'mysql'           # 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
DATABASE_NAME = 'dzenlog'             # Or path to database file if using sqlite3.
DATABASE_USER = 'root'             # Not used with sqlite3.



from django.template import Library, Node, Variable
from django.utils.encoding import smart_str
from django.core.urlresolvers import reverse

from pdb import set_trace

register = Library()

class CallNode(Node):
    def __init__(self, args, kwargs, asvar, func_name=None, obj_name=None, method_name=None):
        self.func_name = func_name
        self.obj_name = obj_name
        self.method_name = method_name
        self.args = args 
        self.kwargs = kwargs
        self.asvar = asvar

    def render(self, context):
        if self.func_name is not None:
            func = self.func_name.resolve(context)
        else:
            obj = self.obj_name.resolve(context)
            #method = self.method_name.resolve(context)
            func = getattr(obj, self.method_name)

        args = [arg.resolve(context) for arg in self.args]
        kwargs = dict([(smart_str(k,'ascii'), v.resolve(context))
                       for k, v in self.kwargs.items()])

        result = func(*args, **kwargs)

        if self.asvar:
            context[self.asvar] = result
            return ''
        else:
            return result

def call(parser, token):
    '''
    Call function or object's method and, optionally,
    place result into the context variable.

    Examples:
        {% call obj.get_absolute_url as abs_url %}
        {% call obj.some_method 42, var='test' %}

    If you don't pass an output variable's name, then result
    will be on the tag's output.
    '''
    bits = token.contents.split(' ')
    if len(bits) < 2:
        raise TemplateSyntaxError("'%s' takes at least one argument"
                                  " (function's name)" % bits[0])

    if '.' in bits[1]:
        obj_name, method_name = bits[1].split('.')
        names = dict(obj_name = Variable(obj_name), method_name = method_name)
    else:
        names = dict(func_name = Variable(bits[1]))

    args = []
    kwargs = {}
    asvar = None

    if len(bits) > 2:
        bits = iter(bits[2:])
        for bit in bits:
            if bit == 'as':
                asvar = bits.next()
                break
            else:
                for arg in bit.split(","):
                    if '=' in arg:
                        k, v = arg.split('=', 1)
                        k = k.strip()
                        kwargs[k] = parser.compile_filter(v)
                    elif arg:
                        args.append(parser.compile_filter(arg))
    return CallNode(args, kwargs, asvar, **names)
call = register.tag(call)




from django.db.models.query import CollectedObjects


def upcast(obj):
    '''Upcast object to it's child.'''

    try:
        return obj._upcast_result
    except AttributeError:
        sub_objects = CollectedObjects()
        obj._collect_sub_objects(sub_objects)
        child = sub_objects.items()[0][1].values()[0]
        setattr(obj, '_upcast_result', child)
        setattr(child, '_upcast_result', child)
        return child


def virtual(func):
    '''Decorator for 'virtual' methods.

       Find a child object and call method against it
       instead os original 'func'.

       Apply this decorator to any method of the base
       class.
    '''

    def wrap(self, *args, **kwargs):
        child_object = upcast(self)
        child_method = getattr(child_object, func.__name__)
        if getattr(child_method, '_is_virtual_wrap', False):
            return func(self, *args, **kwargs)
        return child_method(*args, **kwargs)
    wrap.__doc__  = func.__doc__
    wrap.__name__ = func.__name__
    setattr(wrap, '_is_virtual_wrap', True)
    return wrap



import logging
from datetime import datetime

from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string
from django.core.urlresolvers import reverse, NoReverseMatch

import settings
from pdb import set_trace

if settings.HAS_TAGGING:
    from tagging.fields import TagField
    from tagging.models import Tag


from utils import upcast, virtual

def published(queryset):
    return queryset.filter(publish_at__lte=datetime.now())

class GeneralPost(models.Model):
    author      = models.ForeignKey(User)
    title       = models.CharField(_('Title'), max_length=100)
    slug        = models.SlugField(_('Slug title'), max_length=100, unique=True)
    created_at  = models.DateTimeField(_('Create date'), blank=True, editable=False)
    updated_at  = models.DateTimeField(_('Update date'), blank=True, editable=False)
    publish_at  = models.DateTimeField(_('Publish date'), blank=True, null=True)
    comments_on = models.BooleanField(_('Comments On'), default=True)

    if settings.HAS_TAGGING:
        tags    = TagField()

        def _save_tags(self, *args, **kwargs):
            instance = self._downcast()
            setattr(instance, '_tags_cache', self._tags_cache)
            self._meta.get_field('tags')._save(instance=instance)

        def get_tags(self, *args, **kwargs):
            return Tag.objects.get_for_object(self._downcast())
    else:
        def __init__(self, *args, **kwargs):
            if kwargs.pop('tags', None) is not None:
                logging.getLogger('django_dzenlog').warning('Tagging is not supported, add "tagging" into the INSTALLED_APPS.')
            super(GeneralPost, self).__init__(*args, **kwargs)

    def __unicode__(self):
        return self.title

    def _downcast(self):
        return getattr(self, 'generalpost_ptr', self)

    class Meta:
        verbose_name = _('Post')
        verbose_name_plural = _('Posts')
        ordering = ('-publish_at',)

    @virtual
    def _get_template(self):
        return 'django_dzenlog/generalpost.html'

    def render(self, **kwargs):
        return render_to_string(
                self._get_template(),
                dict(object=upcast(self), settings=settings, **kwargs))

    def render_feed(self, **kwargs):
        return render_to_string(
                self._get_template(),
                dict(object=upcast(self), settings=settings, for_feed=True, **kwargs))

    def get_absolute_url(self):
        try:
            obj = upcast(self)
            return reverse('dzenlog-%s-details' % obj._meta.module_name, kwargs=dict(slug=self.slug))
        except NoReverseMatch:
            return reverse('dzenlog-%s-details' % self._meta.module_name, kwargs=dict(slug=self.slug))

    def save(self):
        today = datetime.today()
        if not self.id:
            self.created_at = today
        self.updated_at = today

        result = super(GeneralPost, self).save()

        if settings.HAS_TAGGING:
            self._save_tags()

        return result




from django.http import Http404
from django.views.generic.list_detail import object_list
from django.contrib.syndication.views import feed as syndication_feed

from tagging.models import Tag, TaggedItem

from models import GeneralPost, published

def get_tagged(slug, queryset = None):
    try:
        tags = Tag.objects.filter(name__in = slug.split('+'))
    except Tag.DoesNotExist:
        raise Http404

    if queryset is None:
        queryset = GeneralPost.objects.all()

    queryset = published(queryset)

    # Magic starts here.
    # I am mangling with queryset's model
    # attribute to make 'tagging' belive
    # that it is operates on GeneralPost type.
    general_queryset = queryset.all()
    general_queryset.model = GeneralPost

    tagged_queryset = TaggedItem.objects.get_by_model(general_queryset, tags)
    tagged_queryset.model = queryset.model
    # End of magic.
    return tagged_queryset, tags


def bytag(request, slug, queryset = None, template_name = None, extra_context = {}):
    queryset, tags = get_tagged(slug, queryset)

    extra_context_ = extra_context.copy()
    extra_context_['tags'] = tags
    extra_context_['tags_slug'] = slug

    kwargs = {
        'queryset': queryset,
        'extra_context': extra_context_,
    }
    if template_name is not None:
        kwargs['template_name'] = template_name

    return object_list(request, **kwargs)

def post_list(request, **kwargs):
    kwargs['queryset'] = published(kwargs['queryset'])
    return object_list(request, **kwargs)


def feed(request, slug, param = None, feed_dict = None):
    url = slug
    if param is not None:
        url += '/' + param
    return syndication_feed(request, url, feed_dict)
